# -*- coding: utf-8 -*-

class ResultTypeError(TypeError):
    '''This exception is raised when the class is not available'''
    
    def __init__(self, classname):
        msg = f"ResultType '{classname}' is not implemented"
        super().__init__(msg)

class ResultAttributeError(AttributeError):
    '''This exception is raised when the attribute is missing'''
    
    def __init__(self, obj, attr):
        msg = f"'{obj.__class__.__name__}' object has no attribute '{attr}'"
        super().__init__(msg)


def get_type(classname):
    '''Get type based on classname'''
    if not classname in TYPES.keys():
        raise ResultTypeError(classname)
    return TYPES[classname]


class ResultType(object):
    '''Result Type object for item generic interface'''

    def __init__(self, json):
        self._json = json

    def __repr__(self):
        return f"<{self.__class__.__name__}> {str(self)} (id: {int(self)})"

    def __getattr__(self, attr):
        if not attr in self._json.keys():
            raise ResultAttributeError(self, attr)
        return self._json[attr]

    def keys(self):
        return self._json.keys()

    def values(self):
        return self._json.values()

    def items(self):
        return self._json.items()

class KernelSetDetails(ResultType):
    '''Kernel set details'''

    def __init__(self, json):
        super().__init__(json)

    def __int__(self):
        return int(self.kernelSetId)

    def __str__(self):
        return self.caption


TYPES = globals()



# -*- coding: utf-8 -*-
import pytest

from webgeocalc import API

from requests import HTTPError
from webgeocalc.vars import API_URL
from webgeocalc.types import ResultAttributeError

@pytest.fixture
def solar_system_kernel():
    return {
        'caption': 'Solar System Kernels',
        'sclkId': '0',
        'description': 'Generic kernels for planets, satellites, and some asteroids covering from 1950-01-01 to 2050-01-01.',
        'kernelSetId': '1',
        'missionId': 'gen',
    }

def test_api_default_url():
    assert API.url == API_URL

def test_kernel_sets(solar_system_kernel):
    kernel = API.kernel_sets()[0]

    assert int(kernel) == int(solar_system_kernel['kernelSetId'])
    assert str(kernel) == solar_system_kernel['caption']
    
    for key in kernel.keys():
        assert key in solar_system_kernel.keys()

    for value in kernel.values():
        assert value in solar_system_kernel.values()
    
    for key, values in kernel.items():
        assert key in solar_system_kernel.keys()
        assert value in solar_system_kernel.values()

    with pytest.raises(ResultAttributeError):
        kernel.wrong_attr


def test_response_err():
    with pytest.raises(HTTPError):
        API.get('/') # 404 error



# -*- coding: utf-8 -*-
import requests
import json

from .vars import API_URL
from .types import get_type

class Api(object):
    '''WebGeoCalc API object'''

    def __init__(self, url=API_URL):
        self.url = url

    def get(self, url):
        '''Parsed get request'''
        response = requests.get(self.url + url)
        if response.ok:
            return self.loads(response.content)
        else:
            response.raise_for_status()

    def loads(self, json_content):
        '''Load json content based on result type'''
        data = json.loads(json_content)
        dtype = get_type(data['resultType'])
        return [dtype(item) for item in data['items']]

    def kernel_sets(self):
        '''Get list of kernel sets available on the server.'''
        return self.get('/kernel-sets')


# Export default API object
API = Api()
